package com.javacodegeeks;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import java.io.IOException;

/**
 * Example code for XPath Inner-text selection
 * @author Java Code Geeks
 */
public class ExampleXPathInnertext {

    private static DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
    private static DocumentBuilder builder = null;
    private static XPath xPath = null;
    private static Document document = null;

    public static void main(String q[]) {
        try {
            builder = builderFactory.newDocumentBuilder();
            document = builder.parse(ExampleXPathInnertext.class.getResourceAsStream("/user.xml"));
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        xPath =  XPathFactory.newInstance().newXPath();
        findParentForGivenInnertext("Over");
        findChildForGivenInnerText("Over");
        findAllChildNodes("Male");
    }

    /*
     * Select an element that has a child with a particular inner-text
     */
    private static void findParentForGivenInnertext(String innerText) {
        try {
            String expression = "//Name[MiddleName='" + innerText + "']";
            NodeList nl = (NodeList) xPath.compile(expression).evaluate(document, XPathConstants.NODESET);
            System.out.println("######### Output for expression: " + expression + " ########");
            System.out.println(nl.item(0).getTextContent());
        } catch (XPathExpressionException e) {
            e.printStackTrace();
        }
    }

    /*
     * Select a child with a particular inner-text
     */
    private static void findChildForGivenInnerText(String innerText) {
        try {
            String expression = "//Name/MiddleName[text() = '" + innerText + "']";
            NodeList nl = (NodeList) xPath.compile(expression).evaluate(document, XPathConstants.NODESET);
            System.out.println("######### Output for expression: " + expression + " ########");
            System.out.println(nl.item(0).getTextContent());
            expression = "//Name/MiddleName[. = '" + innerText + "']";
            NodeList n2 = (NodeList) xPath.compile(expression).evaluate(document, XPathConstants.NODESET);
            System.out.println("######### Output for expression: " + expression + " ########");
            System.out.println(n2.item(0).getTextContent());
        } catch (XPathExpressionException e) {
            e.printStackTrace();
        }
    }

    /*
     * Find all child nodes of "User" where any "Sex" of "User" has $innerText as inner text.
     */
    private static void findAllChildNodes(String innerText) {
        try {
            String expression = "//User[descendant::Sex[text()='" + innerText + "']]";
            NodeList nl = (NodeList) xPath.compile(expression).evaluate(document, XPathConstants.NODESET);
            System.out.println("######### Output for expression: " + expression + " ########");
            System.out.println(nl.item(0).getTextContent());
            System.out.println(nl.item(1).getTextContent());
        } catch (XPathExpressionException e) {
            e.printStackTrace();
        }
    }

}
